#!/usr/bin/env node
'use strict'

const http = require('http')
const https = require('https')
const fs = require('fs')
const url = require('url')

const dest = process.argv[2]
const source = process.argv[3]

new Promise( (resolve) => {
  fs.stat(dest, (err, stat) => {
    if(err || !stat.isFile()) return resolve()
    resolve(stat.size)
  })
})
.then( (size) => {
  function getFile(parsedUrl) {
    const get = url.parse(source).protocol === 'https:' ? https.get : http.get
    get(Object.assign({}, parsedUrl, size ? { headers: { range: `bytes=${size}-` } } : null), (res) => {
      const { statusCode } = res
      if(statusCode === 301 || statusCode === 302) {
        return getFile(url.parse(res.headers.location))
      } else if(statusCode >= 200 && statusCode < 300) {
        return outputResponse(res)
      } else {
        throw new Error(`Error fetching ${source}, response code ${statusCode} received.`)
      }
    })
  }
  function outputResponse(res) {
    let start = false
    if(res.statusCode === 206) {
      try {
        start = parseInt(res.headers['content-range'].split(' ')[1].split('-')[0])
      } catch(e) {}
    }
    const ws = fs.createWriteStream(dest, start ? { start, flags: 'a' } : {})
    res.pipe(ws)
  }
  getFile(url.parse(source))
})
